/***************************************************************************
                          kbeardirlister.cpp  -  description
                             -------------------
    begin                : fre mar 8 2002
    copyright            : (C) 2002 by Bjrn Sahlstrm
    email                : kbjorn@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//////////////////////////////////////////////////////////////////////
// Qt specific include files
#include <qdir.h>
//////////////////////////////////////////////////////////////////////
// KDE specific include files
#include <kmessagebox.h>
#include <klocale.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kuserprofile.h>
#include <kmessagebox.h>
#include <kguiitem.h>
#include <kdirlister.h>
#include <kapplication.h>
#include <kdirwatch.h>
#include <kconfig.h>
#include <kio/slave.h>
#include <kio/scheduler.h>
#include <kparts/part.h>
#include <kparts/componentfactory.h>
#include <ktrader.h>
#include <kdebug.h>
//////////////////////////////////////////////////////////////////////
// Application specific include files
#include "kbeardirlister.h"
#include "../../base/kbearlistjob.h"
#include "../../base/kbeardeletejob.h"
#include "../../base/kbearconnectionmanager.h"

//-----------------------------------------------
KBearDirLister::KBearDirLister( QObject* parent, const char* name, const Connection& c )
	:	QObject( parent, name ),
		m_dirLister( 0L ), m_slave( 0L ), m_listJob( 0L ),
		m_statJob( 0L ), 	m_connection( c ), m_useFilter( false ),
		m_isLocal( false ), m_showingDotFiles( false ),
		m_mime( QString::null ), m_state( DISCONNECTED )
{
	KURL url = m_connection.url();
	m_isLocal = url.isLocalFile();
	m_filterList.setAutoDelete( true );
}
//-----------------------------------------------
KBearDirLister::~KBearDirLister(){
	if( m_dirLister )
		delete m_dirLister;
	if( m_slave )
		connectionManager->closeConnection( (unsigned long)this );
	kdDebug()<<"KBearDirLister::~KBearDirLister()"<<endl;
}
//-----------------------------------------------
void KBearDirLister::openURL( const KURL& url, bool reload ) {
	kdDebug()<<"KBearDirLister::openURL() url="<<url.prettyURL()<<" reload="<<reload<<" state="<<m_state<<endl;
	// this method should only be called for directories
	m_url = url;
	m_state |= LISTING;
	if( m_state & DISCONNECTED ) {
		kdDebug()<<"KBearDirLister::openURL() DISCONNECTED url="<<url.prettyURL()<<" reload="<<reload<<endl;
		openConnection();
	}
	else if( m_state & CONNECTED ) {
		if( m_isLocal ) {
			kdDebug()<<"KBearDirLister::openURL() CONNECTED url="<<url.prettyURL()<<" reload="<<reload<<endl;
			m_dirLister->openURL( m_url, false, reload );
		}
		else {
			listRemoteDir( m_url );
		}
	}
}
//-----------------------------------------------
void KBearDirLister::openConnection() {
	kdDebug()<<"KBearDirLister::openConnection() url="<<m_connection.url().prettyURL()<<endl;
	m_state = m_state & ~DISCONNECTED; 
	slotDisconnectSlaveInfo();
	m_state |= CONNECTING;
	if( m_isLocal ) {
		setDirLister();
		return;
	}
	m_slave = connectionManager->openNewConnection( (unsigned long)this, m_connection );
	if( m_slave ) {
		connect( m_slave, SIGNAL( infoMessage( const QString& ) ),
					this, SIGNAL( infoMessage( const QString& ) ) );
		connect( m_slave, SIGNAL( error( int, const QString&) ),
				this, SLOT( slotSlaveError( int, const QString& ) ) );
//		connect( m_slave, SIGNAL( connected() ), SIGNAL( connected() ) );
		connect( m_slave, SIGNAL( connected() ), this, SLOT( slotSlaveConnected() ) );
		// We reconnect automatically when slave dies
		connect( m_slave, SIGNAL( slaveDied( KIO::Slave* ) ), this, SLOT( slotSlaveDied() ) );
	}
	else {
		m_state = m_state & ~CONNECTING;
		m_state |= DISCONNECTED;
	}
}
//-----------------------------------------------
void KBearDirLister::deleteFiles( const KURL::List& urls, bool shred, bool showProgress ) {
	KIO::Job* job;
	m_state |= DELETING;
	if( m_isLocal ) {
		job = KIO::del( urls, shred, showProgress );
		if( ! job )
			return;
	}
	else {
		job = connectionManager->del( (unsigned long)this ,urls, shred, showProgress );
	}
	connect( job, SIGNAL( infoMessage( KIO::Job*, const QString& ) ),
				this, SLOT( slotInfoMessage( KIO::Job*, const QString& ) ) );

	connect( job, SIGNAL( result( KIO::Job*) ), this, SLOT( slotResult( KIO::Job*) ) );
	connect( job, SIGNAL( result( KIO::Job*) ), SIGNAL( deleteFinished() ) );
	KURL::List::ConstIterator it = urls.begin();
	for( ; it != urls.end(); ++it )
		emit deleteItem( new KFileItem( KFileItem::Unknown, KFileItem::Unknown, (*it) ) );
}
//-----------------------------------------------
void KBearDirLister::statURL( const KURL& url ) {
	// first we need to find out what state we're in
	m_statURL = url;
	m_state |= STATING;
	if( m_state & DISCONNECTED ) {
		kdDebug()<<"KBearDirLister::statURL() DISCONNECTED url="<<url.prettyURL()<<endl;
		openConnection();
	}
	else if( m_state & CONNECTED ) {
		kdDebug()<<"KBearDirLister::statURL() CONNECTED url="<<url.prettyURL()<<endl;
		m_statJob = KIO::stat( m_statURL, false, 0, false );
		if( ! m_isLocal ) { // if remote we need to attach the job to the slave
			connectionManager->attachJob( (unsigned long)this, m_statJob );
		}
		connect( m_statJob, SIGNAL( infoMessage( KIO::Job*, const QString& ) ),
					this, SLOT( slotInfoMessage( KIO::Job*, const QString& ) ) );
		connect( m_statJob, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotResult( KIO::Job* ) ) );
		connect( m_statJob, SIGNAL( redirection( KIO::Job*, const KURL& ) ), this, SLOT( slotStatRedirection( KIO::Job*, const KURL& ) ) );
	}
/*
	if( m_statJob ) {
		disconnect( m_statJob, 0, 0, 0 );
		m_statJob = 0L;
	}
	m_statURL = url;
	if( ! m_isLocal && ( ! m_slave || ! m_slave->isAlive() || ! m_slave->isConnected() ) ) {
		openConnection();
		connect( m_slave, SIGNAL( connected() ), this, SLOT( slotPendingStat() ) );
		return;
	}
	disconnect( m_slave, SIGNAL( connected() ), this, SLOT( slotPendingStat() ) );
	m_statJob = KIO::stat( m_statURL, false, 0, false );
	if( ! m_isLocal ) { // if remote we need to attach the job to the slave
		connectionManager->attachJob( (unsigned long)this, m_statJob );
	}
	connect( m_statJob, SIGNAL( infoMessage( KIO::Job*, const QString& ) ),
				this, SLOT( slotInfoMessage( KIO::Job*, const QString& ) ) );
	connect( m_statJob, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotResult( KIO::Job* ) ) );
	connect( m_statJob, SIGNAL( redirection( KIO::Job*, const KURL& ) ), this, SLOT( slotStatRedirection( KIO::Job*, const KURL& ) ) );
*/
}
//-----------------------------------------------
void KBearDirLister::mkdir( const KURL& newDir ) {
	m_state |= MAKING_DIR;
	KIO::SimpleJob* job = KIO::mkdir( newDir );
	if( ! m_isLocal ) { // if remote we need to attach the job to the slave
		connectionManager->attachJob( (unsigned long)this, job );
	}
	connect( job, SIGNAL( infoMessage( KIO::Job*, const QString& ) ),
				this, SLOT( slotInfoMessage( KIO::Job*, const QString& ) ) );
	connect( job, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotResult( KIO::Job* ) ) );
	connect( job, SIGNAL( result( KIO::Job* ) ), SIGNAL( reload() ) );
}
//-----------------------------------------------
void KBearDirLister::setNameFilter(const QString& nameFilter) {
	kdDebug()<<"KBearDirLister::setNameFilter() filter="<<nameFilter<<endl;
	m_useFilter = ( nameFilter != "*" && nameFilter != "");

	if( ! m_useFilter )
		return;
	m_filterList.clear();
	// Split on white space
	QStringList list = QStringList::split( ' ', nameFilter );
	for( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
		m_filterList.append( new QRegExp(*it, false, true ) );
}
//-----------------------------------------------
void KBearDirLister::setShowingDotFiles( bool show ) {
	m_showingDotFiles = show;
	if( m_isLocal ) {
		m_dirLister->setShowingDotFiles( show );
		m_dirLister->emitChanges();
	}
/*
	else {
		emit clear();
	}
*/
}
//-----------------------------------------------
void KBearDirLister::stop() {
	kdDebug()<<"KBearDirLister::stop()"<<endl;
	if( m_state & GETTING_PREVIEW ) {
		m_previewConnect = "stopped";  // we need this so we don't get stuck in loop waiting for preview connect
	}
	else if( m_state & GETTING_MIME ) {
		m_mime = "stopped";                  // or preview mime
	}
	else if( m_state & LISTING && m_listJob ) {
		disconnect( m_listJob, SIGNAL( infoMessage( KIO::Job*, const QString& ) ), this, 0 );
		disconnect( m_listJob, SIGNAL( result( KIO::Job* ) ), this, 0 );
	}
	else if( m_state & STATING && m_statJob ) {
		disconnect( m_statJob, SIGNAL( infoMessage( KIO::Job*, const QString& ) ), this, 0 );
		disconnect( m_statJob, SIGNAL( result( KIO::Job* ) ), this, 0 );
	}
/*
	m_previewConnect = "stopped";  // we need this so we don't get stuck in loop waiting for preview connect
	m_mime = "stopped";                  // or preview mime
*/
	if( m_isLocal )
		m_dirLister->stop();
	else if( m_slave ) {
/*		if( m_slave->isAlive() )
			m_slave->kill();
*/
		connectionManager->closeConnection( (unsigned long)this );
		emit infoMessage( "internal" + i18n("Connection closed") );
		slotDisconnectSlaveInfo();
	}
/*
	if(  m_listJob )
		disconnect( m_listJob, SIGNAL( infoMessage( KIO::Job *, const QString& ) ), this, 0 );
	if(  m_statJob )
		disconnect( m_statJob, SIGNAL( infoMessage( KIO::Job *, const QString& ) ), this, 0 );
	if(  m_slave )
		disconnect( m_slave, SIGNAL( infoMessage( const QString& ) ), this, 0 );
*/
	m_slave = 0L;
	m_listJob = 0L;
	m_statJob = 0L;
	m_state = DISCONNECTED;
	emit finishedLoading();
}
//-----------------------------------------------
KParts::ReadOnlyPart* KBearDirLister::getPreviewPart( const KURL& url, QWidget* parent ) {
	kdDebug()<<"KBearDirLister::getPreviewPart url="<<url.prettyURL()<<endl;
	m_state |= GETTING_PREVIEW;
	m_previewURL = url;
	m_mime = QString::null;
	m_partViewer = 0L;
	m_partViewerWidget = parent;
	if( m_isLocal ) {
		m_state |= GETTING_MIME;
		m_mime = KMimeType::findByURL( url )->name();
		m_partViewer = getPreviewPartInternal( m_mime );
		kdDebug()<<"KBearDirLister::getPreviewPart local mime="<<m_mime<<" part="<<m_partViewer<<endl;
	}
	else {
		if( ! m_slave || ! m_slave->isAlive() || ! m_slave->isConnected() ) {
			kdDebug()<<"KBearDirLister::getPreviewPart No Slave"<<endl;
			openConnection();
			m_previewConnect = QString::null;
			while( m_previewConnect == QString::null )
				kapp->processEvents();
			if( m_previewConnect == "stopped" ) // if user stopped
				return (KParts::ReadOnlyPart*)1;

		}
		m_state |= GETTING_MIME;
		KIO::TransferJob* job = KIO::get( url, false, false );
		connectionManager->attachJob( (unsigned long)this, job );
		connect( job, SIGNAL( mimetype( KIO::Job*, const QString& ) ),
					this, SLOT( slotMimeType( KIO::Job*, const QString& ) ) );
		connect( job, SIGNAL( infoMessage( KIO::Job*, const QString& ) ),
					this, SLOT( slotInfoMessage( KIO::Job*, const QString& ) ) );
		connect( job, SIGNAL( result( KIO::Job*) ), this, SLOT( slotResult( KIO::Job* ) ) );

		// This is fragile, we will be stuck here until we receive a mime type or an error occure
		while( m_mime == QString::null )
			kapp->processEvents();

		kdDebug()<<"KBearDirLister::getPreviewPart remote mime="<<m_mime<<" part="<<m_partViewer<<endl;
	}
	m_state = m_state & ~GETTING_PREVIEW;
	return m_partViewer;
}
//-----------------------------------------------
// END PUBLIC PART
//-----------------------------------------------
void KBearDirLister::slotDisconnectSlaveInfo() {
	disconnect( m_slave, SIGNAL( infoMessage( const QString& ) ),
				this, SIGNAL( infoMessage( const QString& ) ) );
}
//-----------------------------------------------
void KBearDirLister::slotSlaveConnected() {
	kdDebug()<<"KBearDirLister::slotSlaveConnected()"<<endl;
	disconnect( m_slave, SIGNAL( infoMessage( const QString& ) ),
				this, SIGNAL( infoMessage( const QString& ) ) );
	m_state = m_state & ~CONNECTING;
	m_state = m_state & ~DISCONNECTED;
	m_state |= CONNECTED;
	if( m_state & STATING ) {
		statURL( m_statURL );
	}
	else if( m_state & LISTING ) {
		listRemoteDir( m_url );
	}
	else if( m_state & GETTING_PREVIEW ) {
		m_previewConnect = "connected";
	}
//	disconnect( m_slave, SIGNAL( connected() ), this, SIGNAL( connected() ) );
	disconnect( m_slave, SIGNAL( connected() ), this, SLOT( slotSlaveConnected() ) );
	disconnect( m_slave, SIGNAL( error( int, const QString&) ),
				this, SLOT( slotSlaveError( int, const QString& ) ) );
	emit connected();
}
/*
//-----------------------------------------------
void KBearDirLister::slotPendingPreview() {
	kdDebug()<<"KBearDirLister::slotPendingPreview()"<<endl;
	m_previewConnect = "connected";
}
*/
//-----------------------------------------------
KParts::ReadOnlyPart* KBearDirLister::getPreviewPartInternal( const QString& mime ) {
	kdDebug()<<"KBearDirLister::getPreviewPartInternal() mime in="<<mime<<endl;
	// first we need to find out if user want ReadOnly, ReadWrite or want to be asked
	QString oldGroup = kapp->config()->group();
	kapp->config()->setGroup("General");
	unsigned int viewMode = kapp->config()->readUnsignedNumEntry( "Preview Mode", 0 );

	if( viewMode == 2 ) { // Ask
		int ret = KMessageBox::questionYesNoCancel( 0L, i18n( "Do you want to open the file in read only mode or in read/write mode ?"),
													i18n("Select view mode..."), KGuiItem(i18n("Read only")), KGuiItem(i18n("Read/write")) );
		if( ret == KMessageBox::Cancel )
			m_partViewer = (KParts::ReadOnlyPart*)1;	
		else if( ret == KMessageBox::No )
			viewMode = 1; // ReadWrite						
	}

	QObject* obj = 0L;
	if( viewMode == 1 ) { // ReadWrite
		obj = KParts::ComponentFactory::
							createInstanceFromQuery<KParts::ReadWritePart>
							( mime, QString::null, m_partViewerWidget, "PreviewPart" );
	}
	if( ! obj ) { // ReadOnly
		obj = KParts::ComponentFactory::
							createInstanceFromQuery<KParts::ReadOnlyPart>
							( mime, QString::null, m_partViewerWidget, "PreviewPart" );
	}
	if( obj )
		m_partViewer = dynamic_cast<KParts::ReadOnlyPart*>( obj );

	kapp->config()->setGroup( oldGroup );
	return m_partViewer;
}
/*
//-----------------------------------------------
void KBearDirLister::slotMimeResult( KIO::Job* job ) {
	kdDebug()<<"KBearDirLister::slotMimeResult() job="<<job<<endl;
	if( job->error() ) {
		QStringList list = job->detailedErrorStrings();
		KMessageBox::detailedError( 0L, list[1], list[2], list[0] );
	}
	m_mime = "error";
}
*/
//-----------------------------------------------
void KBearDirLister::slotMimeType( KIO::Job*, const QString& mime ) {
	kdDebug()<<"KBearDirLister::slotMimeType() mime="<<mime<<endl;
	m_partViewer = getPreviewPartInternal( mime );
	if( m_partViewer && (long)m_partViewer > 1 ) {
		connectionManager->putOnHold( (unsigned long)this );
		connect( m_slave, SIGNAL( infoMessage( const QString& ) ),
					this, SIGNAL( infoMessage( const QString& ) ) );
	}
	m_mime = mime;
}
//-----------------------------------------------
void KBearDirLister::slotStatRedirection( KIO::Job*, const KURL& url ) {
	kdDebug()<<"KBearDirLister::slotStatRedirection() url="<<url.prettyURL()<<endl;
	m_statURL = url;
}
//-----------------------------------------------
void KBearDirLister::slotSlaveError( int error , const QString& errStr ) {
	kdDebug()<<"KBearDirLister::slotSlaveError() error="<<error<<" errStr="<<errStr<<endl;
	if( error ) {
		emit gotError( error );
		if( ! errStr.isEmpty() )
			KMessageBox::error( 0L, KIO::buildErrorString( error, errStr) );
	}
	m_state = DISCONNECTED;
}
//-----------------------------------------------
void KBearDirLister::slotSlaveDied() {
	m_state = DISCONNECTED;
}
//-----------------------------------------------
void KBearDirLister::slotResult( KIO::Job* job ) {
	kdDebug()<<"KBearDirLister::slotResult() job="<<job<<" error="<<job->error()<<" state="<<m_state<<endl;
	int error = job->error();
	unsigned int oldState = m_state;
	if( m_state & STATING ) {
		kdDebug()<<"KBearDirLister::slotResult() STATING job="<<job<<" error="<<job->error()<<endl;
		m_state = m_state & ~STATING;		
		const KIO::UDSEntry entry = m_statJob->statResult();
		m_statJob = 0L;
		if( ! error ) {
			if( entry.isEmpty() ) {
				kdDebug()<<"KBearDirLister::slotResult() STATING EMPTY ENTRY"<<endl;
				return;
			}
			KFileItem fileItem( entry, m_statURL );
			if( fileItem.isFile() )
				emit openFile( &fileItem );
			else // if not a file we assume a directory
				emit openDir( &fileItem );
			m_statURL = KURL();
		}
	}
	else if( m_state & LISTING ) {
		kdDebug()<<"KBearDirLister::slotResult() LISTING job="<<job<<" error="<<job->error()<<endl;
		m_state = m_state & ~LISTING;
		m_listJob = 0L;
		emit finishedLoading();
	}
	else if( m_state & GETTING_MIME ) {
		kdDebug()<<"KBearDirLister::slotResult() GETTING_MIME job="<<job<<" error="<<job->error()<<endl;
		m_state = m_state & ~GETTING_MIME;
		m_mime = "error";
	}
	else if( m_state & MAKING_DIR ) {
		kdDebug()<<"KBearDirLister::slotResult() MAKING_DIR job="<<job<<" error="<<job->error()<<endl;
		m_state = m_state & ~MAKING_DIR;
	}
	else if( m_state & DELETING ) {
		kdDebug()<<"KBearDirLister::slotResult() DELETING job="<<job<<" error="<<job->error()<<endl;
		m_state = m_state & ~DELETING;
	}
	if( m_slave && ! m_slave->isAlive() ) {
		if( ! (oldState & DELETING && ! m_connection.singleConnection() ) )
			 m_state = DISCONNECTED;
	}
	if( error ) {
		QStringList list = job->detailedErrorStrings();
		KMessageBox::detailedError( 0L, list[1], list[2], list[0] );
	}
}
/*
//-----------------------------------------------
void KBearDirLister::slotPendingList() {
	kdDebug()<<"KBearDirLister::slotPendingList()"<<endl;
	disconnect( m_slave, SIGNAL( connected() ), this, SLOT( slotPendingList() ) );
	listRemoteDir( m_url );
}
//-----------------------------------------------
void KBearDirLister::slotPendingStat() {
	kdDebug()<<"KBearDirLister::slotPendingStat()"<<endl;
	disconnect( m_slave, SIGNAL( connected() ), this, SLOT( slotPendingStat() ) );
	statURL( m_statURL );
}
*/
//-----------------------------------------------
void KBearDirLister::listRemoteDir( const KURL& url ) {
	kdDebug()<<"KBearDirLister::listRemoteDir() url="<<url.prettyURL()<<endl;
	m_state |= LISTING;
	if( m_state & DISCONNECTED ) {
		openConnection();
	}
	else if( m_state & CONNECTED ) {
		emit clear();
		emit started();
		m_listJob = KBearListJob::listDir( (unsigned long)this, url, false, true );
		connectionManager->attachJob( (unsigned long)this, m_listJob );
		connect( m_listJob, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList & ) ),
					this, SLOT( slotEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
		connect( m_listJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( slotResult( KIO::Job * ) ) );
		connect( m_listJob, SIGNAL( redirection( KIO::Job *, const KURL & ) ),
					this, SLOT( slotRedirection( KIO::Job*, const KURL& ) ) );
		connect( m_listJob, SIGNAL( infoMessage( KIO::Job *, const QString& ) ),
					this, SLOT( slotInfoMessage( KIO::Job *, const QString& ) ) );
		connect( m_listJob, SIGNAL( percent( KIO::Job *, unsigned long ) ),
					this, SLOT( slotPercent( KIO::Job *, unsigned long ) ) );
		connect( m_listJob, SIGNAL( speed( KIO::Job *, unsigned long ) ),
					this, SLOT( slotSpeed( KIO::Job *, unsigned long ) ) );
	}
/*
	if( m_listJob ) {
		disconnect( m_listJob, 0, 0, 0 );
		emit finishedLoading();
		QApplication::sendPostedEvents();
		m_listJob = 0L;
	}
	if( ! m_isLocal && ( ! m_slave || ! m_slave->isAlive() || ! m_slave->isConnected() ) ) {
		openConnection();
		connect( m_slave, SIGNAL( connected() ), this, SLOT( slotPendingList() ) );
		return;
	}
	emit clear();
	emit started();
	m_listJob = KBearListJob::listDir( (unsigned long)this, url, false, true );
	connectionManager->attachJob( (unsigned long)this, m_listJob );
	connect( m_listJob, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList & ) ),
				this, SLOT( slotEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
	connect( m_listJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( slotResult( KIO::Job * ) ) );
	connect( m_listJob, SIGNAL( redirection( KIO::Job *, const KURL & ) ),
				this, SLOT( slotRedirection( KIO::Job*, const KURL& ) ) );
	connect( m_listJob, SIGNAL( infoMessage( KIO::Job *, const QString& ) ),
				this, SLOT( slotInfoMessage( KIO::Job *, const QString& ) ) );
	connect( m_listJob, SIGNAL( percent( KIO::Job *, unsigned long ) ),
				this, SLOT( slotPercent( KIO::Job *, unsigned long ) ) );
	connect( m_listJob, SIGNAL( speed( KIO::Job *, unsigned long ) ),
				this, SLOT( slotSpeed( KIO::Job *, unsigned long ) ) );
*/
}
//-----------------------------------------------
void KBearDirLister::slotRedirection( KIO::Job*, const KURL& url ) {
	slotRedirection( url );
}
//-----------------------------------------------
void KBearDirLister::slotRedirection( const KURL& url ) {
	kdDebug()<<"KBearDirLister::slotRedirection() url="<<url.prettyURL()<<endl;
	emit redirection( url );
	m_url = url;
}
//-----------------------------------------------
void KBearDirLister::slotPercent( KIO::Job*, unsigned long pcnt ) {
	emit progress( static_cast<int>(pcnt) );
}
//-----------------------------------------------
void KBearDirLister::slotSpeed( KIO::Job*, unsigned long s ) {
	emit speed( static_cast<int>(s) );
}
//-----------------------------------------------
void KBearDirLister::slotInfoMessage( KIO::Job*, const QString& message ) {
	emit infoMessage( message );
}
//-----------------------------------------------
void KBearDirLister::slotEntries( KIO::Job* job, const KIO::UDSEntryList& entryList ) {
	kdDebug()<<"KBearDirLister::slotEntries()"<<endl;
	if( job != m_listJob )
		return;
	static const QString& dot = KGlobal::staticQString(".");
	static const QString& dotdot = KGlobal::staticQString("..");

	KURL url = static_cast<KIO::ListJob*>(job)->url();
	
	KIO::UDSEntryListConstIterator it = entryList.begin();
	KIO::UDSEntryListConstIterator end = entryList.end();
	KFileItemList fileItems;
	
	for( ; it != end; ++it ) {
		QString name;

		// find out about the name
		KIO::UDSEntry::ConstIterator entit = (*it).begin();
		for( ; entit != (*it).end(); ++entit ) {
			 if( (*entit).m_uds == KIO::UDS_NAME ) {
				name = (*entit).m_str;
				break;
			}
		}
		Q_ASSERT( !name.isEmpty() );
		if( name.isEmpty() )
			continue;
		if( name != dot && name != dotdot ) {
			KFileItem* item = new KFileItem( *it, url, true, true );
			Q_ASSERT( item );
			if( name.left(1) == "." && ! m_showingDotFiles )
				emit deleteItem( item );
			else
				fileItems.append( item );
		}
	}
	slotNewItems( fileItems );
}
//-----------------------------------------------
void KBearDirLister::slotNewItems( const KFileItemList& items ) {
	kdDebug()<<"KBearDirLister::slotNewItems()"<<endl;
	KFileItemListIterator it( items );
	KFileItemList list;
	QString name;
	for( ; it.current(); ++it ) {
		name = it.current()->name();
		if( ! m_useFilter || matchesNameFilter( name ) )
			list.append( it.current() );
	}
	emit newItems( list );
}
//-----------------------------------------------
bool KBearDirLister::matchesNameFilter( const QString& name ) const {
	for( QPtrListIterator<QRegExp> it( m_filterList ); it.current(); ++it )	{
		if ( it.current()->search( name ) != -1 )
			return true;
	}

	return false;
}
//-----------------------------------------------
void KBearDirLister::setDirLister() {
	if( m_dirLister ) // sanity check
		delete m_dirLister;

	m_dirLister = new KDirLister;
	m_dirLister->setAutoUpdate( true );

	connect( m_dirLister, SIGNAL( percent( int ) ), SIGNAL( progress( int ) ) );
	connect( m_dirLister, SIGNAL( started( const KURL& ) ), SIGNAL( started() ) );
	connect( m_dirLister, SIGNAL( newItems( const KFileItemList &) ), this, SLOT( slotNewItems( const KFileItemList &) ) );
	connect( m_dirLister, SIGNAL( completed() ), SIGNAL(finishedLoading() ) );
	connect( m_dirLister, SIGNAL( canceled() ), SIGNAL(finishedLoading() ) );
	connect( m_dirLister, SIGNAL( deleteItem( KFileItem* ) ), SIGNAL( deleteItem( KFileItem* ) ) );
	connect( m_dirLister, SIGNAL( redirection( const KURL& ) ), this, SLOT( slotRedirection( const KURL& ) ) );
	connect( m_dirLister, SIGNAL( infoMessage( const QString& ) ), SIGNAL( infoMessage( const QString& ) ) );
	connect( m_dirLister, SIGNAL( clear() ), SIGNAL( clear() ) );
	connect( m_dirLister, SIGNAL( speed(int) ), SIGNAL( speed(int) ) );
	m_state = m_state & ~CONNECTING;
	m_state |= CONNECTED;
	emit connected();	
}
//-----------------------------------------------
#ifndef NO_INCLUDE_MOCFILES
#include "kbeardirlister.moc"
#endif
